//---------------------------------------------------------------------------
#include <vcl.h>
#include <registry.hpp>
#pragma hdrstop

#include "WebServer.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TWebServer *)
{
        new TWebServer(NULL);
}
//---------------------------------------------------------------------------
__fastcall TWebServer::TWebServer(TComponent* Owner)
        : TServerSocket(Owner)
{
 FMainDocument="index.html";
 MainDocument="index.html";
 MIMEType="";
 ServerReplyData=new TStringList();
 ReplyHeader=new TStringList();
 ReplyBody=new TStringList();
}

__fastcall TWebServer::~TWebServer()
{
 delete ServerReplyData;
 delete ReplyHeader;
 delete ReplyBody;

}
//---------------------------------------------------------------------------
namespace Webserver
{
        void __fastcall PACKAGE Register()
        {
                 TComponentClass classes[1] = {__classid(TWebServer)};
                 RegisterComponents("Internet", classes, 0);
        }
}
//---------------------------------------------------------------------------


void __fastcall TWebServer::SetMainDocument(AnsiString value)
{
    FMainDocument = value;
}

AnsiString __fastcall TWebServer::GetMainDocument()
{
    return FMainDocument;
}


bool __fastcall TWebServer::ParseHTTPHeader(AnsiString& HTTPHeader, AnsiString& Method)
{
 int i;
 HTTPHeader=HTTPHeader.LowerCase();
 if(HTTPHeader.Pos(METHOD_GET)==0)
 {
   Method=ERR_UNKNOWN_METHOD;
   return false;
 }
 Method=METHOD_GET;
 for(i=0;!HTTPHeader.IsDelimiter("/",i);i++)
  if(i>HTTPHeader.Length())
  {
   Method=ERR_UNKNOWN_METHOD;
   return false;
 }
 RequestDocument=FBasePath;
 RequestDocument+=HTTPHeader.SubString(i+1,HTTPHeader.Pos("http")-i-2);
 if(RequestDocument=="" || RequestDocument==" ")
  RequestDocument+=FMainDocument;
 HTTPVersion=HTTPHeader.SubString(HTTPHeader.Pos("http"),8);
 if(!HTTPHeader.Pos("keep-alive"))
  KeepAlive=true;
 else
  KeepAlive=false;
 ErrorCode=100;
 return true;
}

int __fastcall TWebServer::CreateReplyHTTPData()
{
 TSearchRec SearchRec;
 ServerReplyData->Clear();
 ReplyHeader->Clear();
 GetFileMIMEType();
 if(FindFirst(BasePath+RequestDocument, faAnyFile, SearchRec)!=0)
  ErrorCode=404;
 else
 {
  ErrorCode=200;
  ResourceSize=SearchRec.Size;
 }
 ReplyHeader->Add(HTTPVersion+" "+AnsiString(ErrorCode)+" Reply from server");
 ReplyHeader->Add("Server: VAS Server /1.0");
 ReplyHeader->Add("Content-type: "+MIMEType);
 ReplyHeader->Add("Accept-ranges: none");
 return ErrorCode;
}


bool __fastcall TWebServer::ExecuteServerReply(TCustomWinSocket* ClientSocket)
{

  if(ParseHTTPHeader(ClientSocket->ReceiveText(),Method))
  {
   if(CreateReplyHTTPData()==200)
    SendStreamToClient(ClientSocket);
   else
   {
    switch (ErrorCode)
    {
     case 404: ReplyHeader->Add("Content-length: "+AnsiString(NOT_FOUND.Length()));
               ReplyHeader->Add("");
               ClientSocket->SendText(ReplyHeader->Text+NOT_FOUND);
               break;
          default: ReplyHeader->Add("Content-length: "+AnsiString(UNKNOWN_ERROR.Length()));
                   ReplyHeader->Add("");
                   ClientSocket->SendText(ReplyHeader->Text+UNKNOWN_ERROR);
                   break;
    }
   }
  }
  else
   return false;


  if(!KeepAlive)
   ClientSocket->Close();
  return true;
}

void __fastcall TWebServer::SetBasePath(AnsiString value)
{
    FBasePath = value;
}

AnsiString __fastcall TWebServer::GetBasePath()
{
    return FBasePath;
}


AnsiString __fastcall TWebServer::GetFileMIMEType()
{
  TRegistry* Registry;
  try
  {
   Registry  = new TRegistry();
  }
  catch(...)
  {
   if (MIMEType=="")
     MIMEType="application/octet-stream";
   return MIMEType;
  }
  Registry->RootKey=HKEY_CLASSES_ROOT;
  if (Registry->KeyExists(ExtractFileExt(RequestDocument)))
  {
   Registry->OpenKey(ExtractFileExt(RequestDocument), false);
   MIMEType=Registry->ReadString("Content Type");
   Registry->CloseKey();
  }
  else
   MIMEType="application/octet-stream";
  delete Registry;
  return MIMEType;
}


bool __fastcall TWebServer::SendStreamToClient(TCustomWinSocket* Socket)
{
 TFileStream* Stream = new TFileStream(RequestDocument,fmOpenRead);

 ReplyHeader->Add("Content-length: "+AnsiString(ResourceSize));
 ReplyHeader->Add("");
 Socket->SendText(ReplyHeader->Text);
 return Socket->SendStream(Stream);
}


